@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/align.c,v 1.3 2006/02/10 23:21:18 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/align.c,v 1.24 2009/09/28 21:17:11 william Exp $
  */
 
 #define _INCLUDE_FROM_ALIGN_C_
@@ -58,6 +58,7 @@
 #include "rect.e"
 #include "select.e"
 #include "setup.e"
+#include "stretch.e"
 #include "strtbl.e"
 #include "util.e"
 
@@ -73,38 +74,135 @@ int	vertAlign=ALIGN_T;
 
 int	alignDirectType=INVALID;
 
-void DistrSelObjs()
-{
-   register int i;
-   struct SelRec *sel_ptr, *next_sel;
+static
+void SetSSSIForDistr(obj_ptr, vsel_ptr, update_pmvi)
    struct ObjRec *obj_ptr;
    struct VSelRec *vsel_ptr;
-   int dx=0, dy=0, ltx, lty, rbx, rby, count, w, h;
+   int update_pmvi;
+{
+   IntPoint *vlist=NULL;
+   int i=0, n=0;
+   StretchStructuredSplineInfo *psssi=NULL;
+
+   switch (obj_ptr->type) {
+   case OBJ_POLY:
+      vlist = obj_ptr->detail.p->vlist;
+      n = obj_ptr->detail.p->n;
+      if (obj_ptr->userdata == NULL &&
+            obj_ptr->detail.p->curved == LT_STRUCT_SPLINE) {
+         psssi = (StretchStructuredSplineInfo*)malloc(
+               sizeof(StretchStructuredSplineInfo));
+         if (psssi == NULL) FailAllocMessage();
+         memset(psssi, 0, sizeof(StretchStructuredSplineInfo));
+
+         SetIPTInfoForStretchPoly(vsel_ptr->v_index[0], n, vlist, psssi);
+
+         psssi->orig_abs_x = psssi->new_abs_x = vlist[vsel_ptr->v_index[0]].x;
+         psssi->orig_abs_y = psssi->new_abs_y = vlist[vsel_ptr->v_index[0]].y;
+         obj_ptr->userdata = psssi;
+         if (obj_ptr->ctm != NULL) {
+            psssi->rotated_orig_abs_x = psssi->rotated_new_abs_x =
+                  vsel_ptr->x[0];
+            psssi->rotated_orig_abs_y = psssi->rotated_new_abs_y =
+                  vsel_ptr->y[0];
+         }
+      }
+      if (update_pmvi && vsel_ptr->pmvi == NULL &&
+            obj_ptr->detail.p->curved != LT_STRUCT_SPLINE) {
+         vsel_ptr->pmvi = (MoveVertexInfo*)malloc(
+               vsel_ptr->n*sizeof(MoveVertexInfo));
+         memset(vsel_ptr->pmvi, 0, vsel_ptr->n*sizeof(MoveVertexInfo));
+
+         for (i=0; i < vsel_ptr->n; i++) {
+            vsel_ptr->pmvi[i].orig_abs_x = vsel_ptr->pmvi[i].new_abs_x =
+                  vlist[vsel_ptr->v_index[i]].x;
+            vsel_ptr->pmvi[i].orig_abs_y = vsel_ptr->pmvi[i].new_abs_y =
+                  vlist[vsel_ptr->v_index[i]].y;
+            if (obj_ptr->ctm != NULL) {
+               vsel_ptr->pmvi[i].rotated_orig_abs_x =
+                     vsel_ptr->pmvi[i].rotated_new_abs_x = vsel_ptr->x[i];
+               vsel_ptr->pmvi[i].rotated_orig_abs_y =
+                     vsel_ptr->pmvi[i].rotated_new_abs_y = vsel_ptr->y[i];
+            }
+         }
+      }
+      break;
+   case OBJ_POLYGON:
+      vlist = obj_ptr->detail.g->vlist;
+      n = obj_ptr->detail.g->n;
+      if (obj_ptr->userdata == NULL &&
+            obj_ptr->detail.g->curved == LT_STRUCT_SPLINE) {
+         psssi = (StretchStructuredSplineInfo*)malloc(
+               sizeof(StretchStructuredSplineInfo));
+         if (psssi == NULL) FailAllocMessage();
+         memset(psssi, 0, sizeof(StretchStructuredSplineInfo));
+
+         SetIPTInfoForStretchPoly(vsel_ptr->v_index[0], n, vlist, psssi);
+
+         psssi->orig_abs_x = psssi->new_abs_x = vlist[vsel_ptr->v_index[0]].x;
+         psssi->orig_abs_y = psssi->new_abs_y = vlist[vsel_ptr->v_index[0]].y;
+         obj_ptr->userdata = psssi;
+         if (obj_ptr->ctm != NULL) {
+            psssi->rotated_orig_abs_x = psssi->rotated_new_abs_x =
+                  vsel_ptr->x[0];
+            psssi->rotated_orig_abs_y = psssi->rotated_new_abs_y =
+                  vsel_ptr->y[0];
+         }
+      }
+      if (update_pmvi && vsel_ptr->pmvi == NULL &&
+            obj_ptr->detail.g->curved != LT_STRUCT_SPLINE) {
+         vsel_ptr->pmvi = (MoveVertexInfo*)malloc(
+               vsel_ptr->n*sizeof(MoveVertexInfo));
+         memset(vsel_ptr->pmvi, 0, vsel_ptr->n*sizeof(MoveVertexInfo));
+
+         for (i=0; i < vsel_ptr->n; i++) {
+            vsel_ptr->pmvi[i].orig_abs_x = vsel_ptr->pmvi[i].new_abs_x =
+                  vlist[vsel_ptr->v_index[i]].x;
+            vsel_ptr->pmvi[i].orig_abs_y = vsel_ptr->pmvi[i].new_abs_y =
+                  vlist[vsel_ptr->v_index[i]].y;
+            if (obj_ptr->ctm != NULL) {
+               vsel_ptr->pmvi[i].rotated_orig_abs_x =
+                     vsel_ptr->pmvi[i].rotated_new_abs_x = vsel_ptr->x[i];
+               vsel_ptr->pmvi[i].rotated_orig_abs_y =
+                     vsel_ptr->pmvi[i].rotated_new_abs_y = vsel_ptr->y[i];
+            }
+         }
+      }
+      break;
+   }
+}
+
+void DistrSelObjs()
+{
+   int i=0, dx=0, dy=0, ltx, lty, rbx, rby, count, w, h;
+   struct SelRec *sel_ptr=NULL, *next_sel=NULL;
+   struct ObjRec *obj_ptr=NULL;
+   struct VSelRec *vsel_ptr=NULL;
    double x=0.0, y=0.0, h_dist=0.0, v_dist=0.0;
 
    if ((topSel==NULL && topVSel==NULL) ||
          (horiAlign==ALIGN_N && vertAlign==ALIGN_N)) {
       return;
    }
-
    if (numObjLocked != 0) {
       MsgBox(TgLoadString(STID_CANNOT_DISTRIBUTE_LOCKED_OBJS),
             TOOL_NAME, INFO_MB);
       return;
    }
-
    if (curChoice == VERTEXMODE) {
-      int start, num_pts;
-      struct DistrVRec *dv_ptr, *left_dv, *right_dv, *top_dv, *bottom_dv;
-      struct DistrVRec *ptr, *hori_dv, *vert_dv;
-      struct SelRec *tmp_top_sel, *tmp_bot_sel, *tmp_sel_ptr;
-      IntPoint *v=NULL;
+      int start=0, vertices_count=0;
+      struct DistrVRec *dv_ptr=NULL, *left_dv=NULL, *right_dv=NULL;
+      struct DistrVRec *top_dv=NULL, *bottom_dv=NULL;
+      struct DistrVRec *ptr=NULL, *hori_dv=NULL, *vert_dv=NULL;
+      struct SelRec *tmp_top_sel=NULL, *tmp_bot_sel=NULL, *tmp_sel_ptr=NULL;
+      StretchStructuredSplineInfo *psssi=NULL;
 
-      if ((num_pts=CountSelectedVertices()) <= 2) return;
+      if ((vertices_count=CountSelectedVertices()) <= 2) return;
 
       HighLightReverse();
 
-      dv_ptr = (struct DistrVRec *)malloc(2*num_pts*sizeof(struct DistrVRec));
+      dv_ptr = (struct DistrVRec *)malloc(
+            2*vertices_count*sizeof(struct DistrVRec));
       if (dv_ptr == NULL) FailAllocMessage();
 
       start = 1;
@@ -130,16 +228,19 @@ void DistrSelObjs()
       top_dv = bottom_dv = ptr;
 
       ptr++;
-      for (sel_ptr = topSel; sel_ptr != NULL; sel_ptr = sel_ptr->next) {
+      for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
          sel_ptr->obj->marked = FALSE;
+         sel_ptr->obj->userdata = NULL;
       }
-      for (vsel_ptr = topVSel; vsel_ptr != NULL; vsel_ptr = vsel_ptr->next) {
-         int obj_type = vsel_ptr->obj->type, last_index=0, n;
+      for (vsel_ptr=topVSel; vsel_ptr != NULL; vsel_ptr=vsel_ptr->next) {
+         int obj_type=vsel_ptr->obj->type, last_index=0, n=0;
 
-         if (obj_type == OBJ_POLYGON) last_index = vsel_ptr->obj->detail.g->n-1;
+         if (obj_type == OBJ_POLYGON) {
+            last_index = vsel_ptr->obj->detail.g->n-1;
+         }
          obj_ptr = vsel_ptr->obj;
          n = vsel_ptr->n;
-         for (i = start; i < n; i++) {
+         for (i=start; i < n; i++) {
             if (!(obj_type==OBJ_POLYGON && vsel_ptr->v_index[i]==last_index)) {
                for (hori_dv=left_dv; hori_dv!=NULL; hori_dv=hori_dv->next) {
                   if (hori_dv->x > vsel_ptr->x[i] ||
@@ -208,25 +309,29 @@ void DistrSelObjs()
       if (horiAlign != ALIGN_N) {
          for (ptr=left_dv; ptr->next!=right_dv; ptr=ptr->next) {
             obj_ptr = ptr->next->obj;
+            vsel_ptr = ptr->next->vsel;
             if (!(obj_ptr->type==OBJ_POLYGON &&
                   ptr->next->index==obj_ptr->detail.g->n-1)) {
                obj_ptr->marked = TRUE;
+               SetSSSIForDistr(obj_ptr, vsel_ptr, TRUE);
             }
          }
       }
       if (vertAlign != ALIGN_N) {
          for (ptr=top_dv; ptr->next!=bottom_dv; ptr=ptr->next) {
             obj_ptr = ptr->next->obj;
+            vsel_ptr = ptr->next->vsel;
             if (!(obj_ptr->type==OBJ_POLYGON &&
                   ptr->next->index==obj_ptr->detail.g->n-1)) {
                obj_ptr->marked = TRUE;
+               SetSSSIForDistr(obj_ptr, vsel_ptr, TRUE);
             }
          }
       }
 
       tmp_top_sel = tmp_bot_sel = NULL;
       count = 0;
-      for (sel_ptr = topSel; sel_ptr != NULL; sel_ptr = sel_ptr->next) {
+      for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
          obj_ptr = sel_ptr->obj;
          if (obj_ptr->marked) {
             count++;
@@ -239,8 +344,10 @@ void DistrSelObjs()
       }
       if (horiAlign != ALIGN_N) {
          x = (double)(left_dv->x);
-         h_dist = ((double)(right_dv->x-x))/((double)(num_pts-1));
+         h_dist = ((double)(right_dv->x-x))/((double)(vertices_count-1));
          for (ptr=left_dv; ptr->next!=right_dv; ptr=ptr->next) {
+            int new_x=0;
+
             obj_ptr = ptr->next->obj;
             vsel_ptr = ptr->next->vsel;
 
@@ -248,21 +355,42 @@ void DistrSelObjs()
                   ptr->next->index==obj_ptr->detail.g->n-1) {
                continue;
             }
-            switch (obj_ptr->type) {
-            case OBJ_POLY: v = obj_ptr->detail.p->vlist; break;
-            case OBJ_POLYGON: v = obj_ptr->detail.g->vlist; break;
-            }
             obj_ptr->marked = TRUE;
-            v[ptr->next->index].x = vsel_ptr->x[ptr->next->vindex] =
-                  round(x+h_dist);
+            new_x = round(x+h_dist);
 
+            if (obj_ptr->userdata != NULL) {
+               psssi = (StretchStructuredSplineInfo *)obj_ptr->userdata;
+
+               if (obj_ptr->ctm == NULL) {
+                  psssi->new_abs_x = new_x;
+                  psssi->abs_dx = new_x - psssi->orig_abs_x;
+               } else {
+                  psssi->rotated_new_abs_x = new_x;
+                  psssi->rotated_abs_dx = new_x - psssi->rotated_orig_abs_x;
+               }
+            } else {
+               int vindex=ptr->next->vindex;
+               MoveVertexInfo *pmvi=vsel_ptr->pmvi;
+
+               if (obj_ptr->ctm == NULL) {
+                  pmvi[vindex].new_abs_x = new_x;
+                  pmvi[vindex].abs_dx = new_x - pmvi[vindex].orig_abs_x;
+               } else {
+                  pmvi[vindex].rotated_new_abs_x = new_x;
+                  pmvi[vindex].rotated_abs_dx = new_x -
+                        pmvi[vindex].rotated_orig_abs_x;
+               }
+            }
+            vsel_ptr->x[ptr->next->vindex] = new_x;
             x += h_dist;
          }
       }
       if (vertAlign != ALIGN_N) {
          y = (double)(top_dv->y);
-         v_dist = ((double)(bottom_dv->y-y))/((double)(num_pts-1));
+         v_dist = ((double)(bottom_dv->y-y))/((double)(vertices_count-1));
          for (ptr=top_dv; ptr->next!=bottom_dv; ptr=ptr->next) {
+            int new_y=0;
+
             obj_ptr = ptr->next->obj;
             vsel_ptr = ptr->next->vsel;
 
@@ -270,47 +398,94 @@ void DistrSelObjs()
                   ptr->next->index==obj_ptr->detail.g->n-1) {
                continue;
             }
-            switch (obj_ptr->type) {
-            case OBJ_POLY: v = obj_ptr->detail.p->vlist; break;
-            case OBJ_POLYGON: v = obj_ptr->detail.g->vlist; break;
-            }
             obj_ptr->marked = TRUE;
-            v[ptr->next->index].y = vsel_ptr->y[ptr->next->vindex] =
-                  round(y+v_dist);
+            new_y = round(y+v_dist);
 
+            if (obj_ptr->userdata != NULL) {
+               psssi = (StretchStructuredSplineInfo *)obj_ptr->userdata;
+
+               if (obj_ptr->ctm == NULL) {
+                  psssi->new_abs_y = new_y;
+                  psssi->abs_dy = new_y - psssi->orig_abs_y;
+               } else {
+                  psssi->rotated_new_abs_y = new_y;
+                  psssi->rotated_abs_dy = new_y - psssi->rotated_orig_abs_y;
+               }
+            } else {
+               int vindex=ptr->next->vindex;
+               MoveVertexInfo *pmvi=vsel_ptr->pmvi;
+
+               if (obj_ptr->ctm == NULL) {
+                  pmvi[vindex].new_abs_y = new_y;
+                  pmvi[vindex].abs_dy = new_y - pmvi[vindex].orig_abs_y;
+               } else {
+                  pmvi[vindex].rotated_new_abs_y = new_y;
+                  pmvi[vindex].rotated_abs_dy = new_y -
+                        pmvi[vindex].rotated_orig_abs_y;
+               }
+            }
+            vsel_ptr->y[ptr->next->vindex] = new_y;
             y += v_dist;
          }
       }
-      for (vsel_ptr = topVSel; vsel_ptr != NULL; vsel_ptr = vsel_ptr->next) {
+      for (vsel_ptr=topVSel; vsel_ptr != NULL; vsel_ptr=vsel_ptr->next) {
          obj_ptr = vsel_ptr->obj;
          if (obj_ptr->marked) {
+            IntPoint *vlist=NULL;
+            
+            switch (obj_ptr->type) {
+            case OBJ_POLY: vlist = obj_ptr->detail.p->vlist; break;
+            case OBJ_POLYGON: vlist = obj_ptr->detail.g->vlist; break;
+            }
             if (obj_ptr->type==OBJ_POLYGON) {
                struct PolygonRec *polygon_ptr=obj_ptr->detail.g;
 
                polygon_ptr->vlist[polygon_ptr->n-1].x = polygon_ptr->vlist[0].x;
                polygon_ptr->vlist[polygon_ptr->n-1].y = polygon_ptr->vlist[0].y;
             }
-            AdjObjSplineVs(obj_ptr);
-            switch (obj_ptr->type) {
-            case OBJ_POLY:
-               if (obj_ptr->detail.p->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.p->n,
-                        obj_ptr->detail.p->vlist);
+            if (obj_ptr->userdata != NULL) {
+               StretchStructuredSplineInfo *psssi=
+                     (StretchStructuredSplineInfo *)obj_ptr->userdata;
+
+               if (obj_ptr->ctm == NULL) {
+                  FinishMoveVertexForStretchStructSpline(vsel_ptr,
+                        psssi->abs_dx, psssi->abs_dy, psssi);
                } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.p->intn,
-                        obj_ptr->detail.p->intvlist);
+                  FinishMoveVertexForStretchStructSpline(vsel_ptr,
+                        psssi->rotated_abs_dx, psssi->rotated_abs_dy, psssi);
                }
-               break;
-            case OBJ_POLYGON:
-               if (obj_ptr->detail.g->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.g->n,
-                        obj_ptr->detail.g->vlist);
+               free(psssi);
+               obj_ptr->userdata = NULL;
+            } else {
+               MoveVertexInfo *pmvi=vsel_ptr->pmvi;
+
+               if (obj_ptr->ctm == NULL) {
+                  for (i=0; i < vsel_ptr->n; i++) {
+                     if (pmvi[i].abs_dx != 0 || pmvi[i].abs_dy != 0) {
+                        vlist[vsel_ptr->v_index[i]].x += pmvi[i].abs_dx;
+                        vlist[vsel_ptr->v_index[i]].y += pmvi[i].abs_dy;
+                     }
+                  }
                } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.g->intn,
-                        obj_ptr->detail.g->intvlist);
+                  for (i=0; i < vsel_ptr->n; i++) {
+                     if (pmvi[i].rotated_abs_dx != 0 ||
+                           pmvi[i].rotated_abs_dy != 0) {
+                        int tmp_x=0, tmp_y=0;
+
+                        ReverseTransformPointThroughCTM(
+                              pmvi[i].rotated_new_abs_x-obj_ptr->x,
+                              pmvi[i].rotated_new_abs_y-obj_ptr->y,
+                              obj_ptr->ctm, &tmp_x, &tmp_y);
+                        vlist[vsel_ptr->v_index[i]].x = tmp_x + obj_ptr->x;
+                        vlist[vsel_ptr->v_index[i]].y = tmp_y + obj_ptr->y;
+                     }
+                  }
                }
-               break;
+               free(pmvi);
+               vsel_ptr->pmvi = NULL;
             }
+            AdjObjSplineVs(obj_ptr);
+            UpdPolyOrPolygonBBox(obj_ptr);
          }
       }
       if (count != 0) {
@@ -366,7 +541,7 @@ void DistrSelObjs()
       count = 1;
       w = left_sel->obj->obbox.rbx - left_sel->obj->obbox.ltx;
       h = left_sel->obj->obbox.rby - left_sel->obj->obbox.lty;
-      for (sel_ptr = botSel->prev; sel_ptr != NULL; sel_ptr = sel_ptr->prev) {
+      for (sel_ptr=botSel->prev; sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
          count++;
          obj_ptr = sel_ptr->obj;
          w += obj_ptr->obbox.rbx - obj_ptr->obbox.ltx;
@@ -791,17 +966,17 @@ void AutoJustify(obj_ptr)
 void AlignSelObjs()
 {
    register int x=0, y=0, i;
-   struct SelRec *sel_ptr;
-   struct ObjRec *obj_ptr;
-   struct VSelRec *vsel_ptr;
-   int pivot_x = 0, pivot_y = 0, auto_justifiable;
-   int dx, dy, ltx=0, lty=0, rbx=0, rby=0;
+   struct SelRec *sel_ptr=NULL;
+   struct ObjRec *obj_ptr=NULL;
+   struct VSelRec *vsel_ptr=NULL;
+   int pivot_x=0, pivot_y=0, auto_justifiable=FALSE;
+   int dx=0, dy=0, ltx=0, lty=0, rbx=0, rby=0;
 
    if (topSel == NULL && topVSel == NULL) return;
 
    if (curChoice == VERTEXMODE) {
-      int changed=FALSE, num_pts=0, first_time=TRUE, start;
-      IntPoint *v=NULL;
+      int changed=FALSE, n=0, first_time=TRUE, start;
+      IntPoint *vlist=NULL;
 
       if (topVSel!=NULL && (topVSel->next!=NULL || topVSel->n>=3  ||
             (topVSel->n==2 &&
@@ -843,52 +1018,97 @@ void AlignSelObjs()
       }
 
       for (vsel_ptr = topVSel; vsel_ptr != NULL; vsel_ptr = vsel_ptr->next) {
+         StretchStructuredSplineInfo sssi;
+         int curved=(-1), ssn=(-1);
+         IntPoint *ssvlist=NULL;
+         struct PolyRec *poly_ptr=NULL;
+         struct PolygonRec *polygon_ptr=NULL;
+
+         memset(&sssi, 0, sizeof(StretchStructuredSplineInfo));
          obj_ptr = vsel_ptr->obj;
          switch (obj_ptr->type) {
          case OBJ_POLY:
-            num_pts = obj_ptr->detail.p->n;
-            v = obj_ptr->detail.p->vlist;
+            poly_ptr = obj_ptr->detail.p;
+            curved = poly_ptr->curved;
+            n = poly_ptr->n;
+            vlist = poly_ptr->vlist;
+            if (curved == LT_STRUCT_SPLINE) {
+               ssn = poly_ptr->ssn;
+               ssvlist = poly_ptr->ssvlist;
+               SetIPTInfoForStretchPoly(vsel_ptr->v_index[0], n, vlist, &sssi);
+            }
             break;
          case OBJ_POLYGON:
-            num_pts = obj_ptr->detail.g->n;
-            v = obj_ptr->detail.g->vlist;
+            polygon_ptr = obj_ptr->detail.g;
+            curved = polygon_ptr->curved;
+            n = polygon_ptr->n;
+            vlist = polygon_ptr->vlist;
+            if (curved == LT_STRUCT_SPLINE) {
+               ssn = polygon_ptr->ssn;
+               ssvlist = polygon_ptr->ssvlist;
+               SetIPTInfoForStretchPolygon(vsel_ptr->v_index[0], n, vlist,
+                     &sssi);
+            }
             break;
          }
          PrepareToReplaceAnObj(obj_ptr);
-         for (i = 0; i < vsel_ptr->n; i++) {
+         for (i=0; i < vsel_ptr->n; i++) {
+            int abs_dx=0,  abs_dy=0;
+
             if (horiAlign!=ALIGN_N && horiAlign!=ALIGN_S &&
                   vsel_ptr->x[i]!=pivot_x) {
                changed = TRUE;
-               vsel_ptr->x[i] = pivot_x;
-               v[vsel_ptr->v_index[i]].x = pivot_x;
+               abs_dx = pivot_x - vsel_ptr->x[i];
             }
             if (vertAlign!=ALIGN_N && vertAlign!=ALIGN_S &&
                   vsel_ptr->y[i]!=pivot_y) {
                changed = TRUE;
-               vsel_ptr->y[i] = pivot_y;
-               v[vsel_ptr->v_index[i]].y = pivot_y;
+               abs_dy = pivot_y - vsel_ptr->y[i];
+            }
+            if (changed) {
+               if (obj_ptr->ctm == NULL) {
+                  vsel_ptr->x[i] = pivot_x;
+                  vsel_ptr->y[i] = pivot_y;
+                  if (curved == LT_STRUCT_SPLINE) {
+                     /*
+                      * don't do anything here, will be handled in
+                      *       FinishMoveVertexForStretchStructSpline();
+                      */
+                  } else {
+                     vlist[vsel_ptr->v_index[i]].x = pivot_x;
+                     vlist[vsel_ptr->v_index[i]].y = pivot_y;
+                  }
+               } else {
+                  int tmp_x=0, tmp_y=0, tmp_x2=0, tmp_y2=0;
+
+                  ReverseTransformPointThroughCTM(
+                        pivot_x-obj_ptr->x, pivot_y-obj_ptr->y, obj_ptr->ctm,
+                        &tmp_x, &tmp_y);
+                  if (curved == LT_STRUCT_SPLINE) {
+                     /*
+                      * don't do anything here, will be handled in
+                      *       FinishMoveVertexForStretchStructSpline();
+                      */
+                  } else {
+                     dx = tmp_x + obj_ptr->x - vlist[vsel_ptr->v_index[i]].x;
+                     dy = tmp_y + obj_ptr->y - vlist[vsel_ptr->v_index[i]].y;
+                     vlist[vsel_ptr->v_index[i]].x = tmp_x + obj_ptr->x;
+                     vlist[vsel_ptr->v_index[i]].y = tmp_y + obj_ptr->y;
+                  }
+                  TransformPointThroughCTM(tmp_x, tmp_y, obj_ptr->ctm,
+                         &tmp_x2, &tmp_y2);
+                  vsel_ptr->x[i] = tmp_x2 + obj_ptr->x;     
+                  vsel_ptr->y[i] = tmp_y2 + obj_ptr->y; 
+               }
+            }
+            if (curved == LT_STRUCT_SPLINE) {
+               FinishMoveVertexForStretchStructSpline(vsel_ptr, abs_dx, abs_dy,
+                     &sssi);
             }
          }
          if (changed) {
             AdjObjSplineVs(obj_ptr);
-            switch (obj_ptr->type) {
-            case OBJ_POLY:
-               if (obj_ptr->detail.p->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, num_pts, v);
-               } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.p->intn,
-                        obj_ptr->detail.p->intvlist);
-               }
-               break;
-            case OBJ_POLYGON:
-               if (obj_ptr->detail.g->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, num_pts, v);
-               } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.g->intn,
-                        obj_ptr->detail.g->intvlist);
-               }
-               break;
-            }
+            UpdNonIntSplinePolyBBox(obj_ptr, n, vlist);
             RecordReplaceAnObj(obj_ptr);
          } else {
             AbortPrepareCmd(CMD_REPLACE);
@@ -1009,17 +1229,17 @@ void AlignSelObjs()
 
 void AlignSelToPage()
 {
-   register int i;
-   struct SelRec *sel_ptr;
-   struct ObjRec *obj_ptr;
-   struct VSelRec *vsel_ptr;
-   int pivot_x=0, pivot_y=0, dx, dy, ltx, lty, rbx, rby;
+   int i=0;
+   struct SelRec *sel_ptr=NULL;
+   struct ObjRec *obj_ptr=NULL;
+   struct VSelRec *vsel_ptr=NULL;
+   int pivot_x=0, pivot_y=0, dx=0, dy=0, ltx=0, lty=0, rbx=0, rby=0;
 
    if (topSel == NULL && topVSel == NULL) return;
 
    if (curChoice == VERTEXMODE) {
-      int changed=FALSE, num_pts=0;
-      IntPoint *v=NULL;
+      int changed=FALSE, n=0;
+      IntPoint *vlist=NULL;
 
       if (topVSel!=NULL && (topVSel->next!=NULL || topVSel->n>=3  ||
             (topVSel->n==2 &&
@@ -1031,8 +1251,12 @@ void AlignSelToPage()
             return;
          }
       }
+      for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
+         sel_ptr->obj->userdata = NULL;
+      }
       StartCompositeCmd();
       HighLightReverse();
+
       switch (horiAlign) {
       case ALIGN_L: pivot_x = 0; break;
       case ALIGN_C: pivot_x = paperWidth / 2; break;
@@ -1043,17 +1267,18 @@ void AlignSelToPage()
       case ALIGN_M: pivot_y = paperHeight / 2; break;
       case ALIGN_B: pivot_y = paperHeight; break;
       }
-
-      for (vsel_ptr = topVSel; vsel_ptr != NULL; vsel_ptr = vsel_ptr->next) {
+      for (vsel_ptr=topVSel; vsel_ptr != NULL; vsel_ptr=vsel_ptr->next) {
          obj_ptr = vsel_ptr->obj;
+         SetSSSIForDistr(obj_ptr, vsel_ptr, FALSE);
+
          switch (obj_ptr->type) {
          case OBJ_POLY:
-            num_pts = obj_ptr->detail.p->n;
-            v = obj_ptr->detail.p->vlist;
+            n = obj_ptr->detail.p->n;
+            vlist = obj_ptr->detail.p->vlist;
             break;
          case OBJ_POLYGON:
-            num_pts = obj_ptr->detail.g->n;
-            v = obj_ptr->detail.g->vlist;
+            n = obj_ptr->detail.g->n;
+            vlist = obj_ptr->detail.g->vlist;
             break;
          }
          PrepareToReplaceAnObj(obj_ptr);
@@ -1061,36 +1286,63 @@ void AlignSelToPage()
             if (horiAlign!=ALIGN_N && horiAlign!=ALIGN_S &&
                   vsel_ptr->x[i]!=pivot_x) {
                changed = TRUE;
+               if (obj_ptr->userdata != NULL) {
+                  StretchStructuredSplineInfo *psssi=
+                        (StretchStructuredSplineInfo *)obj_ptr->userdata;
+
+                  if (obj_ptr->ctm == NULL) {
+                     psssi->new_abs_x = pivot_x;
+                     psssi->abs_dx = pivot_x - psssi->orig_abs_x;
+                  } else {
+                     psssi->rotated_new_abs_x = pivot_x;
+                     psssi->rotated_abs_dx = pivot_x -
+                           psssi->rotated_orig_abs_x;
+                  }
+               } else {
+                  vlist[vsel_ptr->v_index[i]].x = pivot_x;
+               }
                vsel_ptr->x[i] = pivot_x;
-               v[vsel_ptr->v_index[i]].x = pivot_x;
             }
             if (vertAlign!=ALIGN_N && vertAlign!=ALIGN_S &&
                   vsel_ptr->y[i]!=pivot_y) {
                changed = TRUE;
+               if (obj_ptr->userdata != NULL) {
+                  StretchStructuredSplineInfo *psssi=
+                        (StretchStructuredSplineInfo *)obj_ptr->userdata;
+
+                  if (obj_ptr->ctm == NULL) {
+                     psssi->new_abs_y = pivot_y;
+                     psssi->abs_dy = pivot_y - psssi->orig_abs_y;
+                  } else {
+                     psssi->rotated_new_abs_y = pivot_y;
+                     psssi->rotated_abs_dy = pivot_y -
+                           psssi->rotated_orig_abs_y;
+                  }
+               } else {
+                  vlist[vsel_ptr->v_index[i]].y = pivot_y;
+               }
                vsel_ptr->y[i] = pivot_y;
-               v[vsel_ptr->v_index[i]].y = pivot_y;
+            }
+            if (obj_ptr->userdata != NULL) {
+               StretchStructuredSplineInfo *psssi=
+                     (StretchStructuredSplineInfo *)obj_ptr->userdata;
+
+               if (changed) {
+                  if (obj_ptr->ctm == NULL) {
+                     FinishMoveVertexForStretchStructSpline(vsel_ptr,
+                           psssi->abs_dx, psssi->abs_dy, psssi);
+                  } else {
+                     FinishMoveVertexForStretchStructSpline(vsel_ptr,
+                           psssi->rotated_abs_dx, psssi->rotated_abs_dy, psssi);
+                  }
+               }
+               free(psssi);
+               obj_ptr->userdata = NULL;
             }
          }
          if (changed) {
             AdjObjSplineVs(obj_ptr);
-            switch (obj_ptr->type) {
-            case OBJ_POLY:
-               if (obj_ptr->detail.p->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, num_pts, v);
-               } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.p->intn,
-                        obj_ptr->detail.p->intvlist);
-               }
-               break;
-            case OBJ_POLYGON:
-               if (obj_ptr->detail.g->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, num_pts, v);
-               } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.g->intn,
-                        obj_ptr->detail.g->intvlist);
-               }
-               break;
-            }
+            UpdNonIntSplinePolyBBox(obj_ptr, n, vlist);
             RecordReplaceAnObj(obj_ptr);
          } else {
             AbortPrepareCmd(CMD_REPLACE);
@@ -1156,18 +1408,38 @@ void AlignSelToGrid()
    HighLightReverse();
    if (curChoice == VERTEXMODE) {
       for (vsel_ptr = topVSel; vsel_ptr != NULL; vsel_ptr = vsel_ptr->next) {
-         int changed=FALSE, num_pts=0;
-         IntPoint *v=NULL;
+         int changed=FALSE, n=0, ssn=(-1), curved=(-1);
+         IntPoint *vlist=NULL, *ssvlist=NULL;
+         struct PolyRec *poly_ptr=NULL;
+         struct PolygonRec *polygon_ptr=NULL;
+         StretchStructuredSplineInfo sssi;
+
+         memset(&sssi, 0, sizeof(StretchStructuredSplineInfo));
 
          obj_ptr = vsel_ptr->obj;
          switch (obj_ptr->type) {
          case OBJ_POLY:
-            num_pts = obj_ptr->detail.p->n;
-            v = obj_ptr->detail.p->vlist;
+            poly_ptr = obj_ptr->detail.p;
+            curved = poly_ptr->curved;
+            n = poly_ptr->n;
+            vlist = poly_ptr->vlist;
+            if (curved == LT_STRUCT_SPLINE) {
+               ssn = poly_ptr->ssn;
+               ssvlist = poly_ptr->ssvlist;
+               SetIPTInfoForStretchPoly(vsel_ptr->v_index[0], n, vlist, &sssi);
+            }
             break;
          case OBJ_POLYGON:
-            num_pts = obj_ptr->detail.g->n;
-            v = obj_ptr->detail.g->vlist;
+            polygon_ptr = obj_ptr->detail.g;
+            curved = polygon_ptr->curved;
+            n = polygon_ptr->n;
+            vlist = polygon_ptr->vlist;
+            if (curved == LT_STRUCT_SPLINE) {
+               ssn = polygon_ptr->ssn;
+               ssvlist = polygon_ptr->ssvlist;
+               SetIPTInfoForStretchPolygon(vsel_ptr->v_index[0], n, vlist,
+                     &sssi);
+            }
             break;
          }
          PrepareToReplaceAnObj(obj_ptr);
@@ -1200,10 +1472,51 @@ void AlignSelToGrid()
             }
             if (dx != 0 || dy != 0) {
                changed = TRUE;
-               vsel_ptr->x[i] += dx;
-               vsel_ptr->y[i] += dy;
-               v[vsel_ptr->v_index[i]].x += dx;
-               v[vsel_ptr->v_index[i]].y += dy;
+               if (ssvlist != NULL) {
+                  /* LT_STRUCT_SPLINE */
+                  if (obj_ptr->ctm == NULL) {
+                     vsel_ptr->x[i] += dx;
+                     vsel_ptr->y[i] += dy;
+                  } else {
+                     int tmp_x=0, tmp_y=0, tmp_x2=0, tmp_y2=0;
+
+                     /*
+                      * looks like of silly here, but it's to fix round-off
+                      *         errors
+                      */
+                     ReverseTransformPointThroughCTM(
+                           vsel_ptr->x[i]+dx-obj_ptr->x,
+                           vsel_ptr->y[i]+dy-obj_ptr->y, obj_ptr->ctm,
+                           &tmp_x, &tmp_y);
+                     TransformPointThroughCTM(tmp_x, tmp_y, obj_ptr->ctm,
+                            &tmp_x2, &tmp_y2);
+                     vsel_ptr->x[i] = tmp_x2 + obj_ptr->x;
+                     vsel_ptr->y[i] = tmp_y2 + obj_ptr->y;
+                  }
+                  FinishMoveVertexForStretchStructSpline(vsel_ptr, dx, dy,
+                        &sssi);
+               } else {
+                  if (obj_ptr->ctm == NULL) {
+                     vlist[vsel_ptr->v_index[i]].x += dx;
+                     vlist[vsel_ptr->v_index[i]].y += dy;
+                     vsel_ptr->x[i] += dx;
+                     vsel_ptr->y[i] += dy;
+                  } else {
+                     int tmp_x=0, tmp_y=0, tmp_x2=0, tmp_y2=0;
+
+                     ReverseTransformPointThroughCTM(
+                           vsel_ptr->x[i]+dx-obj_ptr->x,
+                           vsel_ptr->y[i]+dy-obj_ptr->y, obj_ptr->ctm,
+                           &tmp_x, &tmp_y);
+                     vlist[vsel_ptr->v_index[i]].x = tmp_x+obj_ptr->x;
+                     vlist[vsel_ptr->v_index[i]].y = tmp_y+obj_ptr->y;
+
+                     TransformPointThroughCTM(tmp_x, tmp_y, obj_ptr->ctm,
+                            &tmp_x2, &tmp_y2);
+                     vsel_ptr->x[i] = tmp_x2 + obj_ptr->x;
+                     vsel_ptr->y[i] = tmp_y2 + obj_ptr->y;
+                  }
+               }
             }
          }
          drawOrigX = saved_draw_orig_x;
@@ -1211,24 +1524,7 @@ void AlignSelToGrid()
 
          if (changed) {
             AdjObjSplineVs(obj_ptr);
-            switch (obj_ptr->type) {
-            case OBJ_POLY:
-               if (obj_ptr->detail.p->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, num_pts, v);
-               } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.p->intn,
-                        obj_ptr->detail.p->intvlist);
-               }
-               break;
-            case OBJ_POLYGON:
-               if (obj_ptr->detail.g->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, num_pts, v);
-               } else {
-                  UpdPolyBBox(obj_ptr, obj_ptr->detail.g->intn,
-                        obj_ptr->detail.g->intvlist);
-               }
-               break;
-            }
+            UpdNonIntSplinePolyBBox(obj_ptr, n, vlist);
             RecordReplaceAnObj(obj_ptr);
          } else {
             AbortPrepareCmd(CMD_REPLACE);
@@ -1619,9 +1915,9 @@ TgMenu *CreateAlignToPageDirectMenu(parent_menu, x, y, menu_info,
 
 void CenterAnEndPoint()
 {
-   struct ObjRec *other_obj, *poly_obj;
+   struct ObjRec *other_obj=NULL, *poly_obj=NULL;
    IntPoint *v;
-   int cx, cy, x1, y1, xn, yn, d1, dn, num_pts, index, x, y, tmp_x, tmp_y;
+   int cx, cy, x1, y1, xn, yn, d1, dn, n=0, index, x, y, tmp_x, tmp_y;
 
    if (curChoice != NOTHING) return;
    if (topSel == NULL) {
@@ -1649,20 +1945,20 @@ void CenterAnEndPoint()
    }
    cx = (other_obj->obbox.ltx+other_obj->obbox.rbx)>>1;
    cy = (other_obj->obbox.lty+other_obj->obbox.rby)>>1;
-   num_pts = poly_obj->detail.p->n;
+   n = poly_obj->detail.p->n;
    v = poly_obj->detail.p->vlist;
    if (poly_obj->ctm == NULL) {
       x1 = v[0].x;
       y1 = v[0].y;
-      xn = v[num_pts-1].x;
-      yn = v[num_pts-1].y;
+      xn = v[n-1].x;
+      yn = v[n-1].y;
    } else {
       TransformPointThroughCTM(v[0].x-poly_obj->x, v[0].y-poly_obj->y,
             poly_obj->ctm, &tmp_x, &tmp_y);
       x1 = tmp_x+poly_obj->x;
       y1 = tmp_y+poly_obj->y;
-      TransformPointThroughCTM(v[num_pts-1].x-poly_obj->x,
-            v[num_pts-1].y-poly_obj->y, poly_obj->ctm, &tmp_x, &tmp_y);
+      TransformPointThroughCTM(v[n-1].x-poly_obj->x,
+            v[n-1].y-poly_obj->y, poly_obj->ctm, &tmp_x, &tmp_y);
       xn = tmp_x+poly_obj->x;
       yn = tmp_y+poly_obj->y;
    }
@@ -1673,7 +1969,7 @@ void CenterAnEndPoint()
       x = x1;
       y = y1;
    } else {
-      index = num_pts-1;
+      index = n-1;
       x = xn;
       y = yn;
    }
@@ -1692,11 +1988,16 @@ void CenterAnEndPoint()
          v[index].y = tmp_y+poly_obj->y;
       }
       AdjObjSplineVs(poly_obj);
-      if (poly_obj->detail.p->curved != LT_INTSPLINE) {
-         UpdPolyBBox(poly_obj, num_pts, v);
-      } else {
+      switch (poly_obj->detail.p->curved) {
+      case LT_STRAIGHT:
+      case LT_SPLINE:
+      case LT_STRUCT_SPLINE:
+         UpdPolyBBox(poly_obj, n, v);
+         break;
+      case LT_INTSPLINE:
          UpdPolyBBox(poly_obj, poly_obj->detail.p->intn,
                poly_obj->detail.p->intvlist);
+         break;
       }
       RecordReplaceAnObj(poly_obj);
       ltx = selLtX; lty = selLtY; rbx = selRbX; rby = selRbY;
