@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/arc.c,v 1.3 2006/02/10 23:21:18 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/arc.c,v 1.8 2009/09/28 17:43:48 william Exp $
  */
 
 #define _INCLUDE_FROM_ARC_C_
@@ -640,7 +640,7 @@ void DumpArcPath(FP, ObjPtr, xc, yc, xr, yr, dir, angle1, angle2,
       if (colorDump || !useGray) {
          if (preDumpSetup) PSUseColorPattern();
          fprintf(FP, "      %s\n", gPsCmd[PS_FLATTENPATH]);
-         DumpPatFill(FP, pen, 8, ObjPtr->bbox, 6, TRUE);
+         DumpPatFill(FP, pen, ObjPtr->bbox, 6, TRUE);
       } else {
          fprintf(FP, "      %s\n", gPsCmd[PS_STROKE]);
       }
@@ -727,7 +727,7 @@ void DumpArcObj(FP, ObjPtr)
          DumpArcPSPath(FP,xc,yc,xr,yr,dir,angle1,angle2,TRUE,"   ","      ");
          fprintf(FP, "   %s %s %s\n", gPsCmd[PS_CLOSEPATH],
                gPsCmd[PS_CLIP], gPsCmd[PS_NEWPATH]);
-         DumpPatFill(FP, fill, 8, ObjPtr->bbox, 3, TRUE);
+         DumpPatFill(FP, fill, ObjPtr->bbox, 3, TRUE);
       } else {
          GrayCheck(fill);
          fprintf(FP, "   %s %s\n", GrayStr(fill), gPsCmd[PS_SETGRAY]);
@@ -1289,6 +1289,10 @@ struct ObjRec *CreateArcObj(xc, yc, x1, y1, x2, y2, dir, ltx, lty, w, h,
 
    obj_ptr->type = OBJ_ARC;
    obj_ptr->color = colorIndex;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(obj_ptr->color_str, sizeof(obj_ptr->color_str),
+            colorMenuItems[colorIndex]);
+   }
    obj_ptr->id = objId++;
    obj_ptr->dirty = FALSE;
    obj_ptr->rotation = 0;
@@ -2119,6 +2123,7 @@ void ReadArcObj(FP, Inbuf, ObjPtr)
 
    (*ObjPtr)->type = OBJ_ARC;
    (*ObjPtr)->color = QuickFindColorIndex(*ObjPtr, color_str, &new_alloc, TRUE);
+   UtilStrCpyN((*ObjPtr)->color_str, sizeof((*ObjPtr)->color_str), color_str);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -2131,7 +2136,7 @@ void ReadArcObj(FP, Inbuf, ObjPtr)
       char inbuf[MAXSTRING+1];
       struct XfrmMtrxRec *ctm;
 
-      fgets(inbuf, MAXSTRING, FP);
+      (void)fgets(inbuf, MAXSTRING, FP);
       scanLineNum++;
       InitScan(inbuf, "\t\n, ");
 
