@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.8 2006/06/12 23:46:58 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.25 2009/06/15 19:34:23 cvsps Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -430,7 +430,29 @@
 #define CMDID_SETOBJSHADOWOFFSETS     475
 #define CMDID_ADDOBJSHADOW            476
 #define CMDID_REMOVEOBJSHADOW         477
+#define CMDID_COPYUTF8                478
+#define CMDID_PASTEUTF8               479
+#define CMDID_REDUCETOMOBILEWEBCOLORS 480
+#define CMDID_CREATEPIXMAPFROMSEL     481
+#define CMDID_TOGGLEROTATEPIVOT       482
+#define CMDID_SPECIFYROTATEPIVOT      483
+#define CMDID_RESETROTATEPIVOT        484
+#define CMDID_NEXTPOLYROTATEPIVOT     485
+#define CMDID_MOVEROTATEPIVOTARCCNTR  486
+#define CMDID_MOVEROTATEPIVOTCENTER   487
+#define CMDID_MOVEROTATEPIVOTLT       488
+#define CMDID_MOVEROTATEPIVOTRT       489
+#define CMDID_MOVEROTATEPIVOTLB       490
+#define CMDID_MOVEROTATEPIVOTRB       491
+#define CMDID_MOVEROTATEPIVOTLEFT     492
+#define CMDID_MOVEROTATEPIVOTRIGHT    493
+#define CMDID_MOVEROTATEPIVOTTOP      494
+#define CMDID_MOVEROTATEPIVOTBOTTOM   495
+#define CMDID_CHOOSEROTATEPIVOT       496
+#define CMDID_COLORWHEEL              497
+#define CMDID_EXTENDSEGMENT           498
+#define CMDID_TOGGLETIGHTERSPLINES    499
 
-#define MAXCMDIDS 478
+#define MAXCMDIDS 500
 
 #endif /*_CMDIDS_H_*/
