@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/names.c,v 1.4 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/names.c,v 1.8 2009/08/13 01:14:20 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_NAMES_C_
@@ -165,7 +165,7 @@ void ParseSymPath(path_str)
    for (i=0, s=path_str; *s != '\0'; ) {
       s = ReadPath(s, dir_str);
       UtilTrimBlanks(dir_str);
-      if (dir_str != '\0') i++;
+      if (*dir_str != '\0') i++;
    }
    symPath = (char**)malloc(i*sizeof(char*));
    if (symPath == NULL) FailAllocMessage();
@@ -174,7 +174,7 @@ void ParseSymPath(path_str)
    for (i=0, s=path_str; *s != '\0'; ) {
       s = ReadPath(s, dir_str);
       UtilTrimBlanks(dir_str);
-      if (dir_str != '\0') {
+      if (*dir_str != '\0') {
          symPath[i] = (char*)malloc((MAXPATHLENGTH+1)*sizeof(char));
          if (symPath[i] == NULL) FailAllocMessage();
          strcpy(symPath[i], dir_str);
@@ -382,13 +382,13 @@ DspList *DirListing(Path, ExtStr, OtherExtStr)
          if (ext_str != NULL) free(ext_str);
          return NULL;
       }
-#ifdef __EMX__
+#ifdef __CYGWIN__
       if (path[strlen(path)-1] != DIR_SEP) {
          strcat(path, DIR_SEP_STR);
       }
-#else /* ~__EMX__ */
+#else /* ~__CYGWIN__ */
       strcat(path, DIR_SEP_STR);
-#endif /* __EMX__ */
+#endif /* __CYGWIN__ */
    }
 
 #ifdef VMS
