@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ps.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ps.c,v 1.13 2009/02/08 17:53:45 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_PS_C_
@@ -38,6 +38,7 @@
 #include "file.e"
 #include "msg.e"
 #include "ps.e"
+#include "raster.e"
 #include "util.e"
 
 #define DEF_PS_DICT_COUNT 2
@@ -684,15 +685,16 @@ static char *psColorImageMacro[] = {
 };
 
 static char *psColorPatternMacro[] = {
-  "/tgifpatdict 10 dict def",
+  "/tgifpatdict 12 dict def",
   "",
   "/tgifpatbyte",
   " { currentdict /retstr get exch",
-  "   pat i cellsz mod get put",
+  "   bytesperrow mul",
+  "   pat i cellh mod bytesperrow mul bytesperrow getinterval putinterval",
   " } def",
   "",
   "/tgifpatproc",
-  " { 0 1 widthlim {tgifpatbyte} for retstr",
+  " { 0 1 cells_wd 1 sub {tgifpatbyte} for retstr",
   "   /i i 1 add def",
   " } def",
   "",
@@ -702,11 +704,13 @@ static char *psColorPatternMacro[] = {
   "      /w exch def",
   "      /lty exch def",
   "      /ltx exch def",
-  "      /cellsz exch def",
+  "      /cellh exch def",
+  "      /cellw exch def",
   "      /pat exch def",
   "",
-  "      /widthlim w cellsz div cvi 1 sub def",
-  "      /retstr widthlim 1 add string def",
+  "      /cells_wd w cellw div cvi def",
+  "      /bytesperrow cellw 8 div ceiling cvi def",
+  "      /retstr cells_wd bytesperrow mul string def",
   "      /i 0 def",
   "",
   "      tgiforigctm setmatrix",
@@ -719,39 +723,60 @@ static char *psColorPatternMacro[] = {
   NULL
 };
 
-static char *psPatternMacro[] = {
-  "/pat3 <8000000008000000> def",
-  "/pat4 <8800000022000000> def",
-  "/pat5 <8800220088002200> def",
-  "/pat6 <8822882288228822> def",
-  "/pat7 <aa55aa55aa55aa55> def",
-  "/pat8 <77dd77dd77dd77dd> def",
-  "/pat9 <77ffddff77ffddff> def",
-  "/pat10 <77ffffff77ffffff> def",
-  "/pat11 <7fffffff7fffffff> def",
-  "/pat12 <8040200002040800> def",
-  "/pat13 <40a00000040a0000> def",
-  "/pat14 <ff888888ff888888> def",
-  "/pat15 <ff808080ff080808> def",
-  "/pat16 <f87422478f172271> def",
-  "/pat17 <038448300c020101> def",
-  "/pat18 <081c22c180010204> def",
-  "/pat19 <8080413e080814e3> def",
-  "/pat20 <8040201008040201> def",
-  "/pat21 <8844221188442211> def",
-  "/pat22 <77bbddee77bbddee> def",
-  "/pat23 <c1e070381c0e0783> def",
-  "/pat24 <7fbfdfeff7fbfdfe> def",
-  "/pat25 <3e1f8fc7e3f1f87c> def",
-  "/pat26 <0102040810204080> def",
-  "/pat27 <1122448811224488> def",
-  "/pat28 <eeddbb77eeddbb77> def",
-  "/pat29 <83070e1c3870e0c1> def",
-  "/pat30 <fefdfbf7efdfbf7f> def",
-  "/pat31 <7cf8f1e3c78f1f3e> def",
-  "",
-  NULL
-};
+/*
+ * No longer needed.
+ * Now psDynamicPrintPatternDef() generates these from pixmap data.
+ *
+ * static char *psPatternMacro[] = {
+ *   "/pat3 <8000000008000000> def",
+ *   "/pat4 <8800000022000000> def",
+ *   "/pat5 <8800220088002200> def",
+ *   "/pat6 <8822882288228822> def",
+ *   "/pat7 <aa55aa55aa55aa55> def",
+ *   "/pat8 <77dd77dd77dd77dd> def",
+ *   "/pat9 <77ffddff77ffddff> def",
+ *   "/pat10 <77ffffffddffffff> def",
+ *   "/pat11 <7ffffffff7ffffff> def",
+ *   "/pat12 <8040200002040800> def",
+ *   "/pat13 <40a00000040a0000> def",
+ *   "/pat14 <ff888888ff888888> def",
+ *   "/pat15 <ff808080ff080808> def",
+ *   "/pat16 <f87422478f172271> def",
+ *   "/pat17 <038448300c020101> def",
+ *   "/pat18 <081c22c180010204> def",
+ *   "/pat19 <8080413e080814e3> def",
+ *   "/pat20 <8040201008040201> def",
+ *   "/pat21 <8844221188442211> def",
+ *   "/pat22 <77bbddee77bbddee> def",
+ *   "/pat23 <c1e070381c0e0783> def",
+ *   "/pat24 <7fbfdfeff7fbfdfe> def",
+ *   "/pat25 <3e1f8fc7e3f1f87c> def",
+ *   "/pat26 <0102040810204080> def",
+ *   "/pat27 <1122448811224488> def",
+ *   "/pat28 <eeddbb77eeddbb77> def",
+ *   "/pat29 <83070e1c3870e0c1> def",
+ *   "/pat30 <fefdfbf7efdfbf7f> def",
+ *   "/pat31 <7cf8f1e3c78f1f3e> def",
+ *   "",
+ *   NULL
+ * };
+ */
+
+void psDynamicPrintPatternDef(FP, index)
+   FILE *FP;
+   int index;
+{
+   int b=0, numbytes=0, need_to_free=FALSE;
+   unsigned char *bytes=NULL;
+
+   bytes = (unsigned char *)GetPatternBytes(index, &numbytes, &need_to_free);
+   fprintf(FP, "/pat%d <", index);
+   for (b=0; b < numbytes; b++) {
+      fprintf(FP, "%02hhx", ByteReverse(bytes[b]));
+   }
+   fprintf(FP, "> def\n");
+   if (need_to_free) UtilFree((char*)bytes);
+}
 
 static char *psCenterTextMacro[] = {
   "/TGCTX { dup stringwidth pop 2 div neg 0 rmoveto } def",
@@ -994,9 +1019,15 @@ void DumpPSMacro(FP)
       }
    }
    if (psUsePattern) {
-      for (i=0; psPatternMacro[i] != NULL; i++) {
-         fprintf(FP, "%s\n", psPatternMacro[i]);
+      /*
+       * for (i=0; psPatternMacro[i] != NULL; i++) {
+       *    fprintf(FP, "%s\n", psPatternMacro[i]);
+       * }
+       */
+      for (i=3; i < 32; i++) {
+         psDynamicPrintPatternDef(FP, i);
       }
+      fprintf(FP, "\n");
    }
    if (psUseCenterText) {
       for (i=0; psCenterTextMacro[i] != NULL; i++) {
