@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tidget.c,v 1.3 2006/02/10 23:21:22 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tidget.c,v 1.7 2009/06/10 18:52:44 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_TIDGET_C_
@@ -39,6 +39,7 @@
 
 #include "font.e"
 #include "msg.e"
+#include "raster.e"
 #include "setup.e"
 #include "tdgtbase.e"
 #include "tdgtbmpl.e"
@@ -143,6 +144,26 @@ int IsTdgtNotifyEvent(pti, pXEv, pTdgtNtfy)
    return IsTdgtWindowNotifyEvent(pti->tci.win, pXEv, pTdgtNtfy);
 }
 
+/* --------------------- TidgetDrawFocusRect --------------------- */
+
+void TidgetDrawFocusRect(pti, pswi)
+   TidgetInfo *pti;
+   SimpleWinInfo *pswi;
+{
+   Window win=pti->tci.win;
+   XGCValues values;
+   int x=pswi->x, y=pswi->y, w=pswi->w, h=pswi->h;
+
+   XSetDashes(mainDisplay, revDefaultGC, 0, dashList[8], dashListLength[8]);
+   values.line_style = LineOnOffDash;
+   XChangeGC(mainDisplay, revDefaultGC, GCLineStyle, &values);
+
+   XDrawRectangle(mainDisplay, win, revDefaultGC, x-1, y-1, w+2, h+2);
+
+   values.line_style = LineSolid;
+   XChangeGC(mainDisplay, revDefaultGC, GCLineStyle, &values);
+}
+
 /* --------------------- RedrawTidget --------------------- */
 
 void RedrawTidget(pti)
@@ -509,6 +530,16 @@ Window TidgetSetFocusWindow(win)
    return prev_win;
 }
 
+/* --------------------- TidgetIsFocus --------------------- */
+
+int TidgetIsFocus(pti)
+   TidgetInfo *pti;
+{
+   if (gFocusWindow == None) return FALSE;
+
+   return (pti->tci.win == gFocusWindow);
+}
+
 /* --------------------- TidgetGetFontInfo --------------------- */
 
 void TidgetGetFontInfo(pnFontIndex, pnSzUnit)
@@ -592,11 +623,12 @@ void TidgetManagerResetGC()
 
       values.background = myBgPixel;
       values.foreground = myFgPixel;
+      values.font = defaultFontPtr->fid;
       values.fill_style = FillSolid;
       values.function = GXcopy;
       values.ts_x_origin = values.ts_y_origin = 0;
       XChangeGC(mainDisplay, gTidgetManager.gc,
-            GCBackground | GCForeground | GCFillStyle | GCFunction |
+            GCBackground | GCForeground | GCFont | GCFillStyle | GCFunction |
             GCTileStipXOrigin | GCTileStipYOrigin, &values);
    }
 }
