@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/wb.c,v 1.43 2009/10/02 15:31:24 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/wb.c,v 1.44 2009/10/08 16:46:41 william Exp $
  */
 
 #define _INCLUDE_FROM_WB_C_
@@ -2769,14 +2769,14 @@ int InitWhiteBoard2()
 
    if (!gstWBInfo.do_whiteboard) return TRUE;
 
-#if (defined(PTHREAD) || defined(HAVE_LIBPTHREAD))
+#if (!defined(PTHREAD) && !defined(HAVE_LIBPTHREAD))
    sprintf(gszMsgBox, TgLoadString(STID_NO_PTHREAD_CONTINUE), TOOL_NAME,
          TOOL_NAME, TOOL_NAME);
    MsgBox(gszMsgBox, TOOL_NAME, STOP_MB);
    gstWBInfo.do_whiteboard = FALSE;
    SendCommandToSelf(CMDID_QUIT, 0);
    return FALSE;
-#endif /* (defined(PTHREAD) || defined(HAVE_LIBPTHREAD)) */
+#endif /* (!defined(PTHREAD) && !defined(HAVE_LIBPTHREAD)) */
 
    if (!RMCastInit()) {
       gstWBInfo.do_whiteboard = FALSE;
